<?php
declare(strict_types=1);


namespace Lewisdale\App\Controllers;

use Doctrine\Persistence\ObjectRepository;
use Lewisdale\App\Models\Data\Feed;
use Lewisdale\App\Models\Data\FeedFilter;
use Lewisdale\App\Models\Data\FilterTarget;
use Lewisdale\App\Models\Data\FilterType;
use Lewisdale\App\Models\Repositories\FeedFilterRepository;
use Lewisdale\App\Models\Repositories\FeedRepository;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Log\LoggerInterface;
use Slim\Views\Twig;

class FeedController
{
    public function __construct(
        private readonly Twig $view,
        private readonly LoggerInterface $logger,
        private readonly FeedRepository $feedRepository,
        private readonly FeedFilterRepository $feedFilterRepository,
    ) {}

    public function get(ServerRequestInterface $request, ResponseInterface $response)
    {
        $this->logger->info("FeedController::get() called");
        $feeds = $this->feedRepository->findBy([], ['title' => 'ASC']);
        return $this->view->render($response, 'feed/index.twig.html', ['feeds' => $feeds]);
    }

    public function get_feed(ServerRequestInterface $request, ResponseInterface $response)
    {
        $this->logger->info("FeedController::get_feed() called");
        $feed = $this->feedRepository->find($request->getAttribute('id'));

        if (empty($feed)) {
            return $response->withStatus(404);
        }

        $filtered = $feed->get_filtered_feed();

        $body = $response->getBody();
        $body->write($filtered->asXML());

        $this->feedRepository->save($feed);

        return $response->withHeader('Content-Type', 'application/rss+xml')
            ->withBody($body)
            ->withStatus(200);
    }

    public function create(ServerRequestInterface $request, ResponseInterface $response): ResponseInterface
    {
        $this->logger->info("FeedController::create() called");

        $feed = $this->feedRepository->find($request->getAttribute('id'));

        return $this->view->render($response, 'feed/edit.twig.html', ['FilterTypes' => FilterType::cases(), 'FilterTargets' => FilterTarget::cases(), 'feed' => $feed ]);
    }

    public function save(ServerRequestInterface $request, ResponseInterface $response): ResponseInterface
    {
        $this->logger->info("FeedController::save() called");

        $body = $request->getParsedBody();

        $id = $request->getAttribute('id');

        $feed = $id ? $this->feedRepository->find($request->getAttribute('id')) : new Feed();

        $feed->title = $body['title'];
        $feed->url = $body['url'];

        $this->feedRepository->save($feed);

        foreach ($body['feedFilters'] as $filter) {
            $this->logger->info("Creating filter", ['filter' => $filter, 'post' => $_POST]);

            $f = $filter['id'] ? $this->feedFilterRepository->find($filter['id']) : new FeedFilter(
                FilterTarget::from($filter['target']),
                FilterType::from($filter['type']),
                $filter['value'],
                $feed
            );

            if ($f !== null && $filter['id']) {
                $f->filter = FilterType::from($filter['type']);
                $f->target = FilterTarget::from($filter['target']);
                $f->value = $filter['value'];
            }

            $this->feedFilterRepository->save($f);
        }

        return $response->withStatus(302)->withHeader('Location', "/feed");
    }

    public function delete(ServerRequestInterface $request, ResponseInterface $response): ResponseInterface
    {
        $this->logger->info("FeedController::delete() called");
        $feed = $this->feedRepository->find($request->getAttribute('id'));
        if (empty($feed)) {
            return $response->withStatus(404);
        }

        $this->feedRepository->delete($feed);
        return $response->withStatus(302)->withHeader('Location', '/feed');
    }
}